@@ -1,7 +1,7 @@
 /*
- * Gophernicus - Copyright (c) 2009-2014 Kim Holviala <kim@holviala.com>
+ * Gophernicus - Copyright (c) 2009-2018 Kim Holviala <kimholviala@fastmail.com>
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *     * Redistributions of source code must retain the above copyright
@@ -9,7 +9,7 @@
  *     * Redistributions in binary form must reproduce the above copyright
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -26,11 +26,20 @@
 #define _GOPHERNICUS_H
 
 
+/*
+ * Ignore useless snprintf() format truncation warnings on GCC 7+
+ */
+#if __GNUC__ >= 7
+#pragma GCC diagnostic ignored "-Wformat-truncation"
+#endif
+
+
 /*
  * Features
  */
 #undef  ENABLE_STRICT_RFC1436	/* Follow RFC1436 to the letter */
 #undef  ENABLE_AUTOHIDING	/* Hide manually listed resources from generated menus */
+#define ENABLE_HAPROXY1		/* Autodetect HAproxy/Stunnel proxy protocol v1 */
 
 
 /*
@@ -48,12 +57,13 @@
 #define HAVE_POPEN		/* popen() */
 #undef  HAVE_STRLCPY		/* strlcpy() from OpenBSD */
 #undef  HAVE_SENDFILE		/* sendfile() in Linux & others */
+/* #undef  HAVE_LIBWRAP		   autodetected, don't enable here */
 
 /* Linux */
 #ifdef __linux
 #undef  PASSWD_MIN_UID
 #define PASSWD_MIN_UID 500
-#define HAVE_SENDFILE
+#define _FILE_OFFSET_BITS 64
 #endif
 
 /* Embedded Linux with uClibc */
@@ -80,6 +90,11 @@
 #endif
 #endif
 
+/* AIX */
+#if defined(_AIX)
+#define _LARGE_FILES 1
+#endif
+
 /* Add other OS-specific defines here */
 
 /*
@@ -130,6 +145,10 @@ size_t strlcpy(char *dst, const char *src, size_t siz);
 size_t strlcat(char *dst, const char *src, size_t siz);
 #endif
 
+#ifdef HAVE_LIBWRAP
+#include <tcpd.h>
+#endif
+
 /*
  * Compile-time configuration
  */
@@ -148,6 +167,8 @@ size_t strlcat(char *dst, const char *src, size_t siz);
 #define ERROR		-1
 
 #define MATCH		0
+#define WRAP_DENIED	0
+
 
 /* Gopher filetypes */
 #define TYPE_TEXT	'0'
@@ -181,18 +202,19 @@ size_t strlcat(char *dst, const char *src, size_t siz);
 #define HTTP_USERAGENT	"Unknown gopher client"
 
 /* Defaults for settings */
-#define DEFAULT_HOST	"localhost"
-#define DEFAULT_PORT	70
-#define DEFAULT_TYPE	TYPE_TEXT
-#define DEFAULT_MAP	"gophermap"
-#define DEFAULT_TAG	"gophertag"
-#define DEFAULT_CGI	"/cgi-bin/"
-#define DEFAULT_USERDIR	"public_gopher"
-#define DEFAULT_ADDR	"unknown"
-#define DEFAULT_WIDTH	70
-#define DEFAULT_CHARSET	US_ASCII
-#define MIN_WIDTH	33
-#define MAX_WIDTH	200
+#define DEFAULT_HOST		"localhost"
+#define DEFAULT_PORT		70
+#define DEFAULT_TLS_PORT	0
+#define DEFAULT_TYPE		TYPE_TEXT
+#define DEFAULT_MAP		"gophermap"
+#define DEFAULT_TAG		"gophertag"
+#define DEFAULT_CGI		"/cgi-bin/"
+#define DEFAULT_USERDIR		"public_gopher"
+#define DEFAULT_WIDTH		76
+#define DEFAULT_CHARSET		US_ASCII
+#define MIN_WIDTH		33
+#define MAX_WIDTH		200
+#define UNKNOWN_ADDR		"unknown"
 
 /* Session defaults */
 #define DEFAULT_SESSION_TIMEOUT		1800
@@ -276,6 +298,7 @@ typedef struct {
 	char req_selector[BUFSIZE];
 	char req_realpath[BUFSIZE];
 	char req_query_string[BUFSIZE];
+	char req_search[BUFSIZE];
 	char req_referrer[BUFSIZE];
 	char req_local_addr[64];
 	char req_remote_addr[64];
@@ -296,6 +319,7 @@ typedef struct {
 	char server_host_default[64];
 	char server_host[64];
 	int  server_port;
+	int  server_tls_port;
 
 	char default_filetype;
 	char map_file[64];
@@ -331,15 +355,19 @@ typedef struct {
 	char opt_vhost;
 	char opt_query;
 	char opt_caps;
+	char opt_status;
 	char opt_shm;
 	char opt_root;
+	char opt_proxy;
+	char opt_exec;
+	char opt_personal_spaces;
 	char debug;
 } state;
 
 /* Shared memory for session & accounting data */
 #ifdef HAVE_SHMEM
 
-#define SHM_KEY		0xbeeb0007	/* Unique identifier + struct version */
+#define SHM_KEY		0xbeeb0008	/* Unique identifier + struct version */
 #define SHM_MODE	0600		/* Access mode for the shared memory */
 #define SHM_SESSIONS	256		/* Max amount of user sessions to track */
 
@@ -396,7 +424,7 @@ typedef struct {
 	"mbox","M", \
 	"pdf","d","ps","d","doc","d","ppt","d","xls","d","xlsx","d","docx","d","pptx","d", \
 	"mp3","s","wav","s","mid","s","wma","s","flac","s","ogg","s","aiff","s","aac","s", \
-	"avi",";","mp4",";","mpg",";","mov",";","qt",";","asf",";","mpv",";","m4v",";", \
+	"avi",";","mp4",";","mpg",";","mov",";","qt",";","asf",";","mpv",";","m4v",";","webm",";","ogv",";", \
 	NULL, NULL
 
 /*
