@@ -1,5 +1,5 @@
 /*
- * Gophernicus - Copyright (c) 2009-2014 Kim Holviala <kim@holviala.com>
+ * Gophernicus - Copyright (c) 2009-2018 Kim Holviala <kimholviala@fastmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -179,7 +179,7 @@ void vhostlist(state *st)
 			strftime(timestr, sizeof(timestr), DATE_FORMAT, ltime);
 
 			printf("1%-*.*s   %s        -  \t/;%s\t%s\t%i" CRLF,
-				width, width, buf, timestr, dir[i].name, 
+				width, width, buf, timestr, dir[i].name,
 				dir[i].name, st->server_port);
 		}
 
@@ -226,7 +226,7 @@ char gopher_filetype(state *st, char *file, char magic)
 	fclose(fp);
 
 	/* GIF images */
-	if (sstrncmp(buf, "GIF89a") == MATCH || 
+	if (sstrncmp(buf, "GIF89a") == MATCH ||
 	    sstrncmp(buf, "GIF87a") == MATCH) return TYPE_GIF;
 
 	/* JPEG images */
@@ -269,6 +269,9 @@ int gophermap(state *st, char *mapfile, int depth)
 	FILE *fp;
 	struct stat file;
 	char line[BUFSIZE];
+#ifdef HAVE_POPEN
+	char command[BUFSIZE];
+#endif
 	char *selector;
 	char *name;
 	char *host;
@@ -282,28 +285,47 @@ int gophermap(state *st, char *mapfile, int depth)
 
 	/* Try to figure out whether the map is executable */
 	if (stat(mapfile, &file) == OK) {
-		if ((file.st_mode & S_IXOTH)) exe = TRUE;
+		if ((file.st_mode & S_IXOTH)) {
+#ifdef HAVE_POPEN
+			/* Quote the command in case path has spaces */
+			snprintf(command, sizeof(command), "'%s'", mapfile);
+#endif
+			exe = TRUE;
+		}
 		else exe = FALSE;
 	}
 
-	/* As a fallback let's just feed everything to shell.. */
-	else exe = TRUE;
+	/* This must be a shell include */
+	else {
+#ifdef HAVE_POPEN
+		/* Let's assume the shell command runs as is without quoting */
+		sstrlcpy(command, mapfile);
+#endif
+		exe = TRUE;
+	}
 
 	/* Debug output */
 	if (st->debug) {
-		if (exe) syslog(LOG_INFO, "parsing executable gophermap \"%s\"", mapfile);
+		if (exe) {
+			if (st->opt_exec)
+			 syslog(LOG_INFO, "parsing executable gophermap \"%s\"", mapfile);
+			else
+			 syslog(LOG_INFO, "parsing executable gophermap \"%s\" forbidden by -nx", mapfile);
+		}
 		else syslog(LOG_INFO, "parsing static gophermap \"%s\"", mapfile);
 	}
 
 	/* Try to execute or open the mapfile */
+	if (exe & st->opt_exec) {
 #ifdef HAVE_POPEN
-	if (exe) {
 		setenv_cgi(st, mapfile);
-		if ((fp = popen(mapfile , "r")) == NULL) return OK;
+		if ((fp = popen(command, "r")) == NULL) return OK;
+#else
+		return OK;
+#endif
 	}
 	else
-#endif
-		if ((fp = fopen(mapfile , "r")) == NULL) return OK;
+		if ((fp = fopen(mapfile, "r")) == NULL) return OK;
 
 	/* Read lines one by one */
 	while (fgets(line, sizeof(line) - 1, fp)) {
@@ -321,7 +343,7 @@ int gophermap(state *st, char *mapfile, int depth)
 		if (type == '.') return QUIT;
 
 		/* Print a list of users with public_gopher */
-		if (type == '~') {
+		if (type == '~' && st->opt_personal_spaces) {
 #ifdef HAVE_PASSWD
 			userlist(st);
 #endif
@@ -383,7 +405,7 @@ int gophermap(state *st, char *mapfile, int depth)
 		/* Parse port */
 		port = st->server_port;
 		if ((c = strchr(host, '\t'))) {
-			*c = '\0'; 
+			*c = '\0';
 			port = atoi(c + 1);
 		}
 
@@ -411,7 +433,7 @@ int gophermap(state *st, char *mapfile, int depth)
 
 	/* Clean up & return */
 #ifdef HAVE_POPEN
-	if (exe) pclose(fp);
+	if (exe & st->opt_exec) pclose(fp);
 	else
 #endif
 		fclose(fp);
@@ -468,7 +490,7 @@ void gopher_menu(state *st)
 			/* Read & output gophertag */
 			if ((fp = fopen(pathname , "r"))) {
 
-				fgets(buf, sizeof(buf), fp);
+				if (fgets(buf, sizeof(buf), fp) == NULL) strclear(buf);
 				chomp(buf);
 
 				info(st, buf, TYPE_TITLE);
@@ -573,7 +595,7 @@ void gopher_menu(state *st)
 				/* Use the gophertag as displayname */
 				if ((fp = fopen(buf , "r"))) {
 
-					fgets(buf, sizeof(buf), fp);
+					if (fgets(buf, sizeof(buf), fp) == NULL) strclear(buf);
 					chomp(buf);
 					fclose(fp);
 
