@@ -1,5 +1,5 @@
 /*
- * Gophernicus - Copyright (c) 2009-2014 Kim Holviala <kim@holviala.com>
+ * Gophernicus - Copyright (c) 2009-2018 Kim Holviala <kimholviala@fastmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,6 @@ void get_shm_session(state *st, shm_state *shm)
 	/* Get session data */
 	if (st->opt_vhost) {
 		sstrlcpy(st->server_host, shm->session[i].server_host);
-		st->server_port = shm->session[i].server_port;
 	}
 }
 #endif
@@ -108,7 +107,8 @@ void update_shm_session(state *st, shm_state *shm)
 
 	/* Get referrer from old session data */
 	if (*shm->session[i].server_host) {
-		snprintf(buf, sizeof(buf), "gopher://%s:%i/%c%s",
+		snprintf(buf, sizeof(buf), "gopher%s://%s:%i/%c%s",
+			(shm->session[i].server_port == st->server_tls_port ? "s" : ""),
 			shm->session[i].server_host,
 			shm->session[i].server_port,
 			shm->session[i].req_filetype,
